@@ -51,6 +51,8 @@ struct _WnckPagerPrivate
   guint dragging : 1;
 
   GdkPixbuf *bg_cache;
+
+  int layout_manager_token;
 };
 
 enum
@@ -143,6 +145,7 @@ wnck_pager_init (WnckPager *pager)
   pager->priv->orientation = GTK_ORIENTATION_HORIZONTAL;
   pager->priv->workspace_size = 48;
   pager->priv->bg_cache = NULL;
+  pager->priv->layout_manager_token = WNCK_NO_MANAGER_TOKEN;
 }
 
 static void
@@ -216,6 +219,12 @@ wnck_pager_realize (GtkWidget *widget)
 
   widget->style = gtk_style_attach (widget->style, widget->window);
   gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
+
+  pager->priv->layout_manager_token =
+    wnck_screen_try_set_workspace_layout (pager->priv->screen,
+                                          pager->priv->layout_manager_token,
+                                          pager->priv->n_rows,
+                                          0);
 }
 
 static void
@@ -226,6 +235,9 @@ wnck_pager_unrealize (GtkWidget *widget)
   pager = WNCK_PAGER (widget);
 
   wnck_pager_clear_drag (pager);
+
+  wnck_screen_release_workspace_layout (pager->priv->screen,
+                                        pager->priv->layout_manager_token);
   
   GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 }
@@ -310,17 +322,18 @@ wnck_pager_size_request  (GtkWidget      *widget,
       else
 	{
 	  int n_spaces, i, w;
+	  WnckScreen *screen;
 	  PangoLayout *layout;
 
 	  n_spaces = wnck_screen_get_workspace_count (pager->priv->screen);
 	  layout = gtk_widget_create_pango_layout  (widget, NULL);
-	  
+	  screen = pager->priv->screen;
 	  other_dimension_size = 1;
 	  
 	  for (i = 0; i < n_spaces; i++)
 	    {
 	      pango_layout_set_text (layout,
-				     wnck_workspace_get_name (wnck_workspace_get (i)),
+				     wnck_workspace_get_name (wnck_screen_get_workspace (screen, i)),
 				     -1);
 	      pango_layout_get_pixel_size (layout, &w, NULL);
 	      other_dimension_size = MAX (other_dimension_size, w);
@@ -645,7 +658,8 @@ wnck_pager_draw_workspace (WnckPager    *pager,
   if (pager->priv->display_mode == WNCK_PAGER_DISPLAY_CONTENT)
     {      
       windows = get_windows_for_workspace_in_bottom_to_top (pager->priv->screen,
-							    wnck_workspace_get (workspace));
+							    wnck_screen_get_workspace (pager->priv->screen,
+										       workspace));
       
       tmp = windows;
       while (tmp != NULL)
@@ -675,13 +689,16 @@ wnck_pager_draw_workspace (WnckPager    *pager,
   else
     {
       /* Workspace name mode */
+      const char *workspace_name;
       PangoLayout *layout;
       int w, h;
 
       
-      
+
+      workspace_name = wnck_workspace_get_name (wnck_screen_get_workspace (pager->priv->screen,
+									   workspace));
       layout = gtk_widget_create_pango_layout  (GTK_WIDGET (pager),
-						wnck_workspace_get_name (wnck_workspace_get (workspace)));
+						workspace_name);
       
       pango_layout_get_pixel_size (layout, &w, &h);
       
@@ -779,8 +796,8 @@ wnck_pager_expose_event  (GtkWidget      *widget,
 }
 
 static gboolean
-wnck_pager_button_press  (GtkWidget      *widget,
-                          GdkEventButton *event)
+wnck_pager_button_press (GtkWidget      *widget,
+                         GdkEventButton *event)
 {
   WnckPager *pager;
   int i;
@@ -800,7 +817,7 @@ wnck_pager_button_press  (GtkWidget      *widget,
 
       if (POINT_IN_RECT (event->x, event->y, rect))
         {
-          WnckWorkspace *space = wnck_workspace_get (i);
+          WnckWorkspace *space = wnck_screen_get_workspace (pager->priv->screen, i);
 
 	  if (event->button == 1)
 	    handled = TRUE;
@@ -909,7 +926,7 @@ wnck_pager_button_release (GtkWidget        *widget,
 
       if (i >= 0)
 	{
-	  space = wnck_workspace_get (i);
+	  space = wnck_screen_get_workspace (pager->priv->screen, i);
 
 	  if (space)
             {
@@ -931,18 +948,20 @@ wnck_pager_button_release (GtkWidget        *widget,
 
       if (i >= 0)
 	{
-	  space = wnck_workspace_get (i);
+	  space = wnck_screen_get_workspace (pager->priv->screen, i);
 
-	  if (space &&
-	      space != wnck_screen_get_active_workspace (pager->priv->screen))
-	    wnck_workspace_activate (space);
+	  if (space)
+            {
+              if (space != wnck_screen_get_active_workspace (pager->priv->screen))
+                wnck_workspace_activate (space);
+              else if (pager->priv->drag_window)
+                wnck_window_activate (pager->priv->drag_window);
+            }
 	}
       
       if (pager->priv->drag_window)
-	{
-	  wnck_window_activate (pager->priv->drag_window);
-	  wnck_pager_clear_drag (pager);
-	}
+        wnck_pager_clear_drag (pager);
+
       handled = TRUE;
     }
 
@@ -996,6 +1015,12 @@ wnck_pager_set_n_rows (WnckPager *pager,
 
   pager->priv->n_rows = n_rows;
   gtk_widget_queue_resize (GTK_WIDGET (pager));
+  
+  pager->priv->layout_manager_token =
+    wnck_screen_try_set_workspace_layout (pager->priv->screen,
+                                          pager->priv->layout_manager_token,
+                                          pager->priv->n_rows,
+                                          0);
 }
 
 void
@@ -1325,6 +1350,9 @@ wnck_pager_get_background (WnckPager *pager,
   if (pager->priv->screen == NULL)
     return NULL;
 
+  /* FIXME this just globally disables the thumbnailing feature */
+  return NULL;
+  
 #define MIN_BG_SIZE 10
   
   if (width < MIN_BG_SIZE || height < MIN_BG_SIZE)
