@@ -67,7 +67,7 @@ struct _WnckTask
   GtkWidget *image;
   GtkWidget *label;
   
-  gboolean is_application;
+  gboolean is_application; /* TRUE if this task is a grouped application */
   WnckApplication *application;
   WnckWindow *window;
 
@@ -123,6 +123,9 @@ struct _WnckTasklistPrivate
 
   int *size_hints;
   int size_hints_len;
+
+  gint minimum_width;
+  gint minimum_height;
 };
 
 
@@ -360,6 +363,9 @@ wnck_tasklist_init (WnckTasklist *tasklist)
   
   tasklist->priv->grouping = WNCK_TASKLIST_AUTO_GROUP;
   tasklist->priv->grouping_limit = DEFAULT_GROUPING_LIMIT;
+
+  tasklist->priv->minimum_width = DEFAULT_WIDTH;
+  tasklist->priv->minimum_height = DEFAULT_HEIGHT;
 }
 
 static void
@@ -470,6 +476,56 @@ wnck_tasklist_set_grouping_limit (WnckTasklist *tasklist,
   gtk_widget_queue_resize (GTK_WIDGET (tasklist));
 }
 
+/* set the minimum width 
+ * use -1 to unset it (resulting in the default width */
+void 
+wnck_tasklist_set_minimum_width (WnckTasklist *tasklist, gint size)
+{
+  g_return_if_fail (WNCK_IS_TASKLIST (tasklist));
+
+  if (size == -1) size = DEFAULT_WIDTH;
+
+  if (tasklist->priv->minimum_width == size)
+    return;
+
+  tasklist->priv->minimum_width = size;
+  gtk_widget_queue_resize (GTK_WIDGET (tasklist));
+}
+ 
+/* get the minimum width */
+gint
+wnck_tasklist_get_minimum_width (WnckTasklist *tasklist)
+{
+  g_return_val_if_fail (WNCK_IS_TASKLIST (tasklist), 0);
+	
+  return tasklist->priv->minimum_width;
+}
+
+/* set the minimum height
+ * use -1 to unset it (resulting in the default height */
+void 
+wnck_tasklist_set_minimum_height (WnckTasklist *tasklist, gint size)
+{
+  g_return_if_fail (WNCK_IS_TASKLIST (tasklist));
+
+  if (size == -1) size = DEFAULT_HEIGHT;
+
+  if (tasklist->priv->minimum_height == size)
+    return;
+
+  tasklist->priv->minimum_height = size;
+  gtk_widget_queue_resize (GTK_WIDGET (tasklist));
+}
+ 
+/* get the minimum height */
+gint
+wnck_tasklist_get_minimum_height (WnckTasklist *tasklist)
+{
+  g_return_val_if_fail (WNCK_IS_TASKLIST (tasklist), 0);
+	
+  return tasklist->priv->minimum_height;
+}
+
 /* returns the maximal possible button width (i.e. if you
  * don't want to stretch the buttons to fill the alloctions
  * the width can be smaller) */
@@ -645,8 +701,8 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
 
   gtk_widget_get_size_request (widget, &u_width, &u_height);
 
-  requisition->width = DEFAULT_WIDTH;
-  requisition->height = DEFAULT_HEIGHT;
+  requisition->width = tasklist->priv->minimum_width;
+  requisition->height = tasklist->priv->minimum_height;
   
   if (u_height != -1)
     {
@@ -1298,7 +1354,8 @@ wnck_tasklist_activate_task_window     (WnckTask *task)
 
       active_ws = wnck_screen_get_active_workspace (tasklist->priv->screen);
       window_ws = wnck_window_get_workspace (task->window);
-      if (active_ws != window_ws &&
+      if (window_ws &&
+          active_ws != window_ws &&
 	  !tasklist->priv->switch_workspace_on_unminimize)
 	wnck_workspace_activate (window_ws);
 	  
@@ -1313,6 +1370,12 @@ wnck_tasklist_activate_task_window     (WnckTask *task)
 	}
       else
 	{
+          WnckWorkspace *window_ws;
+          
+          window_ws = wnck_window_get_workspace (task->window);
+          if (window_ws)
+            wnck_workspace_activate (window_ws);
+
 	  wnck_window_activate (task->window);
 	}
     }
@@ -1329,7 +1392,8 @@ wnck_tasklist_activate_task_window     (WnckTask *task)
 
 
 static void
-wnck_task_popup_menu (WnckTask  *task)
+wnck_task_popup_menu (WnckTask *task,
+                      gboolean  action_submenu)
 {
   GtkWidget *menu;
   WnckTask *win_task;
@@ -1378,11 +1442,15 @@ wnck_task_popup_menu (WnckTask  *task)
 	}
       
       gtk_widget_show (menu_item);
-      
-      g_signal_connect_object (G_OBJECT (menu_item), "activate",
-                               G_CALLBACK (wnck_task_menu_activated),
-                               G_OBJECT (win_task),
-                               0);      
+
+       if (action_submenu)
+         gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item),
+                                    wnck_create_window_action_menu (win_task->window));
+       else
+         g_signal_connect_object (G_OBJECT (menu_item), "activate",
+                                  G_CALLBACK (wnck_task_menu_activated),
+                                  G_OBJECT (win_task),
+                                  0);
       
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
       
@@ -1414,7 +1482,7 @@ wnck_task_button_toggled (GtkButton *button,
 
   if (task->is_application)
     {
-      wnck_task_popup_menu (task);
+      wnck_task_popup_menu (task, FALSE);
     }
   else
     {
@@ -1647,24 +1715,14 @@ wnck_task_button_press_event (GtkWidget	      *widget,
 			      gpointer         data)
 {
   WnckTask *task = WNCK_TASK (data);
-  
-  if (event->button != 1)
-    {
-      g_signal_stop_emission_by_name (widget,
-				      "button_press_event");
-    }
-  else
-    {
-      if (task->is_application)
-	{
-	  wnck_task_popup_menu (task);
-	  g_signal_stop_emission_by_name (widget,
-					  "button_press_event");
-	}
-    }
 
-  if (!task->is_application &&
-      event->button == 3)
+  if (task->is_application)
+    {
+      wnck_task_popup_menu (task,
+                            event->button == 3);
+      return TRUE;
+    }
+  else if (event->button == 3)
     {
       if (task->action_menu)
         gtk_widget_destroy (task->action_menu);
